/***************************************************************************
                          dcclient.cpp  -  description
                             -------------------
    begin                : Thu Oct 4 2001
    copyright            : (C) 2001 by Mathias Kster
    email                : mathen@ketelhot.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <stdlib.h>

#include <qapplication.h>
#include <qpainter.h>
#include <qprinter.h>
#include <qtextedit.h>
#include <qlineedit.h>
#include <qlistview.h>
#include <qlistbox.h>
#include <qcheckbox.h>
#include <qregexp.h>
#include <qspinbox.h>
#include <qcombobox.h>
#include <qlcdnumber.h>
#include <qpopupmenu.h>
#include <qcursor.h>
#include <qfile.h>
#include <qmessagebox.h>
#include <qlabel.h>
#include <qpushbutton.h>
#include <qlistbox.h>
#include <qtabwidget.h>
#include <qinputdialog.h>
#include <qfiledialog.h>
#include <qclipboard.h>
#include <qsound.h>
#include <qlayout.h>
#include <qsplitter.h>

#include <dchubsearch.h>
#include <dcspy.h>
#include <dctransferview.h>
#include <dcconfig.h>
#include <dcfiletool.h>
#include <dcchat.h>
#include <dcmenuhandler.h>
#include <dcwidget.h>

#include "dcclient.h"

#include "res/connected.xpm"
#include "res/notconnected.xpm"
#include "res/message.xpm"

#include <dclib/cfilemanager.h>
#include <dclib/dcos.h>

/** */
DCClient::DCClient(QWidget* parent, const char *name, int wflags) : DCDialogClient(parent, name, wflags)
{
	int idx;

	m_pWorkspace = (QWorkspace*)parent;

	// set default icon
	setIcon( QPixmap( notconnected_xpm ) );

	// set the ColumnWidthMode to manual, we will take care of this task
	for( idx = 0; idx < ListView_USERLIST->columns(); idx++ )
	{
		ListView_USERLIST->setColumnWidthMode( idx, QListView::Manual );
	}

	ListView_USERLIST->setColumnAlignment(4,Qt::AlignRight);

	m_pMessageList  = new CList<CObject>();
	m_pUserViewList = new CStringList();
	m_pChatList     = new CStringList();

	m_bUseTabWidget = Config->GetShowChatInTab();

	InitDocument();

	lChatFloodCounter = 0;
}

/** */
DCClient::~DCClient()
{
	CObject * Object;

	Disconnect();

	Timer.stop();

	Thread.Lock();

	if ( m_pMessageList )
	{
		while ( (Object = m_pMessageList->Next(0)) != 0 )
		{
			m_pMessageList->Del(Object);
		}

		delete m_pMessageList;
		m_pMessageList = 0;
	}

	Thread.UnLock();
	
	if ( m_pChatList )
	{
		CString s;
		DCChat * chat = 0;

		while( m_pChatList->Next( s, (CObject *&)chat ) )
		{
			m_pChatList->Remove(s);
			chat = 0;
		}

		delete m_pChatList;
		m_pChatList = 0;
	}

	delete m_pUserViewList;
	m_pUserViewList = 0;
}

/** */
void DCClient::InitDocument()
{
	StringMap * map;

	// restore settings
	if ( Config->GetMap("CLIENTVIEW",map) == TRUE )
	{
		if ( ((*map)["WIDTH"].toInt() > 0) && ((*map)["HEIGTH"].toInt() > 0) )
		{
			setGeometry( x(), y(), (*map)["WIDTH"].toInt(), (*map)["HEIGTH"].toInt() );
		}
	}

	// add hubchat widget
	m_pHubChat = new DCChat( TabWidget_CHAT, "HUB-Chat", WDestructiveClose, this, FALSE );

	TabWidget_CHAT->insertTab( m_pHubChat, tr("Hub") );

	// remove chat list tab
	if ( m_bUseTabWidget )
	{
		QWidget * page = TabWidget_CHAT->page(0);
		TabWidget_CHAT->removePage(page);
		delete page;
	}
	else
	{
		// set width mode
		ListView_CHATUSERLIST->setColumnWidthMode( 0, QListView::Maximum );
		// hide the column header
		ListView_CHATUSERLIST->header()->hide();

		connect( ListView_CHATUSERLIST,SIGNAL(doubleClicked(QListViewItem*)), this, SLOT(slotDoubleClickedChatUserList(QListViewItem*)) );
	}

	if ( Config->GetUserListRightAlignment() == TRUE )
		Splitter_CLIENT->moveToFirst(TabWidget_CHAT);

	connect( ListView_USERLIST,SIGNAL(doubleClicked(QListViewItem*)), this, SLOT(slotDoubleClickedUserList(QListViewItem*)) );
//	connect( ListView_USERLIST,SIGNAL(rightButtonClicked( QListViewItem *, const QPoint &, int )), this, SLOT(slotRightButtonClickedUserList(QListViewItem*, const QPoint &, int )) );
	connect( ListView_USERLIST,SIGNAL(contextMenuRequested( QListViewItem *, const QPoint &, int )), this, SLOT(slotRightButtonClickedUserList(QListViewItem*, const QPoint &, int )) );
	connect( TabWidget_CHAT,SIGNAL(currentChanged(QWidget*)), this, SLOT(slotTabWidgetChatCurrentChange(QWidget*)) );

	TabWidget_CHAT->installEventFilter(this);

	connect( &Timer, SIGNAL(timeout()), this, SLOT(timerDone()) );
	
	Timer.start( 500, TRUE );
}

/** */
void DCClient::DeInitDocument()
{
	DCChat * chat = 0;
	StringMap * map;

	// save client view settings
	Config->GetMap("CLIENTVIEW",map);

	(*map)["WIDTH"]  = QString().setNum(width());
	(*map)["HEIGTH"] = QString().setNum(height());

	if ( m_pChatList )
	{
		while( m_pChatList->Next( (CObject *&)chat ) )
		{
			chat->removeEventFilter(this);
			chat->close();
		}
	}
}

/** event filter */
bool DCClient::eventFilter(QObject* object, QEvent* event)
{
	if((event->type() == QEvent::Close)&&((DCClient*)object!=this))
	{
		QCloseEvent* e=(QCloseEvent*)event;
		DCChat* pView=(DCChat*)object;

		CloseChat(pView);

		e->accept();
	}

	return QWidget::eventFilter( object, event );    // standard event processing
}

/** current tab widget change slot */
void DCClient::slotTabWidgetChatCurrentChange( QWidget * w )
{
	if ( ! TabWidget_CHAT->tabIconSet(w).isNull() )
	{
		TabWidget_CHAT->setTabIconSet( w, QIconSet() );
		// TODO: reset nick color
		// SetNickColor(nick,Qt::black);
	}
}

/** overridden so that the columns are resized on show() */
void DCClient::show()
{
	QWidget::show();

	ResizeListViewColumn();
}

/** resize event handler */
void DCClient::resizeEvent( QResizeEvent * )
{
	ResizeListViewColumn();
}

/** resize the ListView columns */
void DCClient::ResizeListViewColumn()
{
	int width;

	if ( ListView_USERLIST->isVisible() )
	{
		width = ListView_USERLIST->width();

		ListView_USERLIST->setColumnWidth( 0, ((width*6)/20) );
		ListView_USERLIST->setColumnWidth( 1, ((width*5)/20) );
		ListView_USERLIST->setColumnWidth( 2, ((width*2)/20) );
		ListView_USERLIST->setColumnWidth( 3, ((width*3)/20) );

		if ( ListView_USERLIST->verticalScrollBar()->isVisible() )
			ListView_USERLIST->setColumnWidth( 4, ((width*4)/20)-ListView_USERLIST->verticalScrollBar()->width()-4 );
		else
			ListView_USERLIST->setColumnWidth( 4, ((width*4)/20) );
	}
}

/** */
void DCClient::UpdateStatusBar()
{
	TextLabel_USERCOUNT->setText( QString().setNum(ListView_USERLIST->childCount()) );
	TextLabel_SHARESIZE->setText( DCFileTool::GetSizeString(GetHubShareSize()).Data() );
}

/** */
void DCClient::timerDone()
{
	CObject * Object;
	CDCMessage *DCMsg;
	int i;

	for(i=0;i<50;i++)
	{
		if ( Thread.TryLock() == FALSE )
		{
			break;
		}

		if ( m_pMessageList != 0 )
		{
			Object = m_pMessageList->Next(0);

			if ( Object != 0 )
			{
				m_pMessageList->Remove(Object);
			}
		}
		else
		{
			Object = 0;
		}

		Thread.UnLock();

		if ( Object == 0 )
		{
			break;
		}

		DCMsg = (CDCMessage*) Object;

		switch ( DCMsg->m_eType )
		{
			case DC_MESSAGE_CONNECTION_STATE:
			{
				CMessageConnectionState *msg = (CMessageConnectionState*) Object;

				if ( msg->eState == estCONNECTED )
				{
					lChatFloodCounter = 0;
					bUserPassword     = FALSE;

					setCaption( ("[+] " + GetHubName()).Data());
					setIcon( QPixmap( connected_xpm ) );
					// repaint new icon in minimized mode
					if ( isMinimized() )
						showMinimized();

					m_pHubChat->AddStatus(tr("Connected").ascii());

					// init userlist
					DC_NickList(0);

					Config->PlaySound(eusCONNECT);
				}
				else if ( msg->eState == estCONNECTIONTIMEOUT )
				{
					m_pHubChat->AddStatus(tr("Connection timeout").ascii());
				}
				else if ( msg->eState == estSOCKETERROR )
				{
					QString s;
					s = tr("Error: '") + msg->sMessage.Data() + "'";
					m_pHubChat->AddStatus(s.ascii());
				}
				else if ( msg->eState == estDISCONNECTED )
				{
					lChatFloodCounter = 0;

					setCaption( ("[-] " + GetHubName()).Data());
					setIcon( QPixmap( notconnected_xpm ) );
					// repaint new icon in minimized mode
					if ( isMinimized() )
						showMinimized();

					m_pHubChat->AddStatus(tr("Disconnected").ascii());
					Config->PlaySound(eusDISCONNECT);
				}

				break;
			}

			case DC_MESSAGE_VALIDATEDENIDE:
			{
				m_pHubChat->AddStatus(tr("Validate denide").ascii());
				Disconnect();
				break;
			}

			case DC_MESSAGE_NICKLIST:
			{
				DC_NickList( (CMessageNickList *) Object );
				break;
			}

			case DC_MESSAGE_OPLIST:
			{
				DC_OpList( (CMessageNickList *) Object );
				break;
			}

			case DC_MESSAGE_REVCONNECTTOME:
			{
				CMessageRevConnectToMe * msg = (CMessageRevConnectToMe*) Object;

				if ( GetMode() == ecmPassive )
				{
					if ( Config->GetSendMessageOnActiveModeRequest() == TRUE )
					{
						SendPrivateMessage( GetNick(), msg->sRemoteNick, tr("Sorry, client works in passive mode").ascii() );
					}
				}

				break;
			}

			case DC_MESSAGE_CHAT:
			{
				CMessageChat * msg = (CMessageChat *) Object;
				
				if ( pChatFloodMessage.sMessage == msg->sMessage && pChatFloodMessage.sNick == msg->sNick )
				{
					// increment flood counter
					lChatFloodCounter++;
				}
				else
				{
					// update message and reset counter
					pChatFloodMessage.sMessage = msg->sMessage;
					pChatFloodMessage.sNick = msg->sNick;
					lChatFloodCounter = 0;
				}

				// if flood detect enabled ?
				if ( Config->GetFloodCount() > 0 )
				{
					if ( lChatFloodCounter >= Config->GetFloodCount() )
					{
						// only one flood message
						if ( lChatFloodCounter == Config->GetFloodCount() )
						{
							m_pHubChat->AddStatus(tr("Flood Detected ("+QString().setNum(lChatFloodCounter)+")").ascii());

							// now the admin kick the user if enabled
							if ( (Config->GetFloodOpKick() == TRUE) && (!IsAdmin(msg->sNick)) && (IsAdmin()) )
							{
								//if OP kick the user for Flooding
								CString kickmessage=Config->GetFloodOpKickMessage();
								SendPrivateMessage( GetNick(), msg->sNick.Data(), ("You Are Being kicked Because: " + kickmessage));
								SendChat( GetNick(), GetNick() + " is kicking " + msg->sNick.Data() + " Because: " + kickmessage );
								SendKick(msg->sNick.Data());
							}
						}

						// don't show the flood messages
						msg = 0;
					}
				}

				if ( msg )
				{
					DC_Chat( msg );
				}

				break;
			}

			case DC_MESSAGE_MYINFO:
			{
				DC_MyInfo( (CMessageMyInfo *) Object );
				break;
			}

			case DC_MESSAGE_QUIT:
			{
				DC_Quit( (CMessageQuit *) Object );
				break;
			}

			case DC_MESSAGE_SEARCH:
			{
				if ( pSpy )
					pSpy->DC_Search( (CMessageSearch *) Object );
				break;
			}

			case DC_MESSAGE_HELLO:
			{
				CMessageHello * msg = (CMessageHello *) Object;

				DC_Hello(msg);

				break;
			}

			case DC_MESSAGE_FORCEMOVE:
			{
				CMessageForceMove * msg = (CMessageForceMove*) Object;

				if ( Config->GetForceMoveEnabled() == TRUE )
				{
					setCaption(msg->sHost.Data());

					m_pHubChat->AddStatus( (tr("Redirect to ")+ msg->sHost.Data() + ":" + QString().setNum(msg->iPort)).ascii() );
				}
				else
				{
					m_pHubChat->AddStatus( (tr("Redirect disabled ")+ msg->sHost.Data() + ":" + QString().setNum(msg->iPort)).ascii() );
				}

				break;
			}

			case DC_MESSAGE_HUBNAME:
			{
				CMessageHubName * msg = (CMessageHubName*) Object;

				if ( (msg->sOldHubName != "") && (msg->sOldHubName != msg->sHubName) )
				{
					m_pHubChat->AddStatus( (tr("Hubname change:")+(" '"+msg->sOldHubName+"' -> '"+msg->sHubName+"'").Data()).ascii() );
				}

				setCaption( ("[+] " + msg->sHubName).Data());

				break;
			}

			case DC_MESSAGE_PRIVATECHAT:
			{
				CMessagePrivateChat * msg = (CMessagePrivateChat*) Object;

				DC_PrivateChat( msg );

				break;
			}

			case DC_MESSAGE_GETPASS:
			{
				bool ok = FALSE;
				DCConfigHubProfile pConfigHubProfile;

				if ( bUserPassword == FALSE )
				{
					// get password from profile
					if ( Config->GetBookmarkHubProfile( GetHubName(), GetHost(), &pConfigHubProfile ) == TRUE )
					{
						if ( pConfigHubProfile.sPassword != "" )
						{
							m_pHubChat->AddStatus(tr("Use password from profile").ascii());
							SendPass(pConfigHubProfile.sPassword);
							bUserPassword = TRUE;
							break;
						}
						else
						{
							m_pHubChat->AddStatus(tr("Password from profile is empty").ascii());
						}
					}
					else
					{
						m_pHubChat->AddStatus(tr("No profile found for this hub").ascii());
					}
				}

				QString text = QInputDialog::getText(
					tr( "Password - ")+GetHubName().Data(),
					tr( "Please enter your password" ),
					QLineEdit::Password, QString::null, &ok, this );

				if ( ok && !text.isEmpty() )
				{
					SendPass(text.ascii());
				}
				else
				{
					Disconnect();
				}

				break;
			}

			case DC_MESSAGE_BADPASS:
			{
				m_pHubChat->AddStatus(tr("Wrong password").ascii());
				break;
			}

			default:
			{
				break;
			}
		}

		if ( Object )
			delete Object;
	}

	Timer.start( 500, TRUE );
}

/** */
int DCClient::DC_CallBack( CObject * Object )
{
	int err;

	Thread.Lock();

	if ( Object != NULL )
	{
		if ( m_pMessageList != NULL )
		{
			m_pMessageList->Add(Object);
			err = 0;
		}
		else
		{
			err = -1;
		}
	}
	else
	{
		err = -1;
	}

	Thread.UnLock();

	return err;
}

/** chat */
void DCClient::DC_Chat( CMessageChat * MessageChat )
{
	m_pHubChat->AddMessage(MessageChat->sNick.Data(),MessageChat->sMessage.Data(),TRUE);

	if ( !m_pHubChat->isVisible() )
	{
		TabWidget_CHAT->setTabIconSet( m_pHubChat, QIconSet( QPixmap(message_xpm) ) );
	}
}

/** myinfo */
void DCClient::DC_MyInfo( CMessageMyInfo * MessageMyInfo )
{
	CString s;
	DCFileTool FileTool;
	QPixmap puser;
	eUserIconType type;

	DCUserItem * pui=0;
	if ( m_pUserViewList->Get( MessageMyInfo->sNick, (CObject*&)pui ) == -1 )
	{
		pui = new DCUserItem();
		m_pUserViewList->Add(MessageMyInfo->sNick,pui);

		if ( MessageMyInfo->eAwayMode == euamNORMAL )
			type = euitUSER;
		else
			type = euitUSER_AWAY;

		pui->pItem = new DC_QCheckListItem( ListView_USERLIST, MessageMyInfo->sNick.Data(), Config->GetUserIcon(type) );
	}
	else if ( pui->MyInfo.bOperator == FALSE )
	{
		if ( pui->MyInfo.eAwayMode != MessageMyInfo->eAwayMode )
		{
			if ( MessageMyInfo->eAwayMode == euamNORMAL )
				type = euitUSER;
			else
				type = euitUSER_AWAY;

			pui->pItem->setPixmap(0,Config->GetUserIcon(type));
		}
	}

	pui->MyInfo = *MessageMyInfo;

	s = MessageMyInfo->sComment;

	if ( Config->GetShowVersionTag() )
		s+= MessageMyInfo->sVerComment;

	pui->pItem->setText(1,s.Data());
	pui->pItem->setText(2,MessageMyInfo->sSpeed.Data());
	pui->pItem->setText(3,MessageMyInfo->sEMail.Data());

	pui->pItem->myvalue = MessageMyInfo->lShared;
	pui->pItem->mycol   = 4;

	s = FileTool.GetSizeString(MessageMyInfo->lShared);
	pui->pItem->setText(4,s.Data());

	UpdateStatusBar();
}

/** hello */
void DCClient::DC_Hello( CMessageHello * MessageHello )
{
	DCChat * chat;
	DCUserItem * pui=0;

	if ( m_pUserViewList->Get( MessageHello->sNick, (CObject*&)pui ) == -1 )
	{
		pui = new DCUserItem();
		m_pUserViewList->Add(MessageHello->sNick,pui);

		pui->pItem = new DC_QCheckListItem( ListView_USERLIST, MessageHello->sNick.Data(), Config->GetUserIcon(euitUSER) );
		pui->MyInfo.sNick = MessageHello->sNick;
		pui->MyInfo.bOperator = FALSE;
		pui->MyInfo.eAwayMode = euamNORMAL;
	}

	pui->pItem->mycol = 4;

	if ( m_pChatList->Get( MessageHello->sNick, (CObject*&)chat ) == 0 )
	{
		chat->AddStatus(tr("User rejoin the hub.").ascii());
	}

	UpdateStatusBar();
}

/** quit */
void DCClient::DC_Quit( CMessageQuit * MessageQuit )
{
	DCChat * chat;
	DCUserItem * pui=0;

	if ( m_pUserViewList->Get( MessageQuit->sNick, (CObject*&)pui ) == 0 )
	{
		ListView_USERLIST->takeItem(pui->pItem);
		delete pui->pItem;
		m_pUserViewList->Del(MessageQuit->sNick);
	}

	if ( m_pChatList->Get( MessageQuit->sNick, (CObject*&)chat ) == 0 )
	{
		chat->AddStatus(tr("User left the hub.").ascii());
        }

	UpdateStatusBar();
}

/** nicklist */
void DCClient::DC_NickList( CMessageNickList * MessageNickList )
{
	QString s = "";
	CString * Nick = 0;

	ListView_USERLIST->setUpdatesEnabled(FALSE);
	ListView_USERLIST->clear();

	m_pUserViewList->Clear();

	if ( MessageNickList )
	{
		while ( (Nick=MessageNickList->NickList.Next(Nick)) != 0 )
		{
			DCUserItem * pui = new DCUserItem();
			m_pUserViewList->Add(*Nick,pui);

			pui->pItem = new DC_QCheckListItem( ListView_USERLIST, (*Nick).Data(), Config->GetUserIcon(euitUSER) );
			pui->pItem->mycol = 4;
			pui->MyInfo.sNick = (*Nick);
			pui->MyInfo.bOperator = FALSE;
			pui->MyInfo.eAwayMode = euamNORMAL;
		}
	}

	ListView_USERLIST->setUpdatesEnabled(TRUE);
	ListView_USERLIST->triggerUpdate();

	UpdateStatusBar();
}

/** oplist */
void DCClient::DC_OpList( CMessageNickList * MessageOpList )
{
	CString * Nick = 0;

	ListView_USERLIST->setUpdatesEnabled(FALSE);

	while ( (Nick=MessageOpList->NickList.Next(Nick)) != 0 )
	{
		DCUserItem * pui=0;
		if ( m_pUserViewList->Get( *Nick, (CObject*&)pui ) == -1 )
		{
			pui = new DCUserItem();
			m_pUserViewList->Add(*Nick,pui);

			pui->pItem = new DC_QCheckListItem( ListView_USERLIST, (*Nick).Data(), Config->GetUserIcon(euitOPERATOR) );
			pui->pItem->mycol = 4;
			pui->MyInfo.sNick = (*Nick);
		}
		else
		{
			if ( pui->MyInfo.bOperator == FALSE )
				pui->pItem->setPixmap( 0, Config->GetUserIcon(euitOPERATOR) );
		}

		pui->MyInfo.bOperator = TRUE;
		pui->MyInfo.eAwayMode = euamNORMAL;
	}

	ListView_USERLIST->setUpdatesEnabled(TRUE);
	ListView_USERLIST->triggerUpdate();

	UpdateStatusBar();
}

void DCClient::DC_PrivateChat( QString nick, QString fromnick, QString message, bool bLocal )
{
	CMessagePrivateChat msg;

	msg.sFrom     = nick.ascii();
	msg.sFromNick = fromnick.ascii();
	msg.sString   = message.ascii();

	DC_PrivateChat( &msg, bLocal );
}

/** */
void DCClient::DC_PrivateChat( CMessagePrivateChat * msg, bool bLocal )
{
	bool bShowChat = TRUE;

	DCChat * chat;

	if ( msg->sFrom == "" )
	{
		return;
	}

	// create a new chat window
	if( m_pChatList->Get( msg->sFrom, (CObject*&)chat ) != 0 )
	{
		if ( m_bUseTabWidget )
		{
			chat = new DCChat( TabWidget_CHAT, "chat", WDestructiveClose, this );
		}
		else
		{
			chat = new DCChat( m_pWorkspace, "chat", WDestructiveClose, this );
			new QListViewItem(ListView_CHATUSERLIST,msg->sFrom.Data());
		}

		// fix hide if chat maximized
		chat->show();
		chat->hide();

		chat->installEventFilter(this);
		chat->SetNick(msg->sFrom.Data(),GetHubName().Data());

		m_pChatList->Add( msg->sFrom, (CObject*)chat );

		if ( (Config->GetOpenPrivateChatWindow() == FALSE) && (bLocal == FALSE) )
		{
			bShowChat = FALSE;
		}
		// check for max lines and set chat invisible
		else if ( (Config->GetChatMessageMaxLines() > 0) &&
			  (QString(msg->sString.Data()).contains("\n") > Config->GetChatMessageMaxLines()) )
		{
			bShowChat = FALSE;
		}

		if ( m_bUseTabWidget )
		{
			TabWidget_CHAT->insertTab( chat, msg->sFrom.Data() );
		}

		if ( bShowChat || bLocal )
		{
			if ( m_bUseTabWidget )
				TabWidget_CHAT->showPage(chat);
			else
				chat->setEnabled(TRUE);
			chat->show();
		}
		else
		{
			if ( !m_bUseTabWidget )
			{
				chat->setEnabled(FALSE);
				chat->hide();
			}
		}

		Config->PlaySound(eusFIRSTRECEIVE);
	}
	// enable all local (user input)
	else if ( bLocal == TRUE )
	{
		if ( chat->isVisible() == FALSE )
		{
			if ( m_bUseTabWidget )
				TabWidget_CHAT->showPage(chat);
			else
				chat->setEnabled(TRUE);
			chat->show();
		}
		SetNickColor(msg->sFrom.Data(),Qt::black);
	}

	if ( msg->sString != "" )
	{
		chat->AddMessage( msg );

		if ( chat->isVisible() == FALSE )
		{
			SetNickColor( msg->sFrom.Data(), Qt::red );

			if ( m_bUseTabWidget )
				TabWidget_CHAT->setTabIconSet( chat, QIconSet( QPixmap(message_xpm) ) );
		}
		else
		{
			SetNickColor( msg->sFrom.Data(), Qt::black );
		}
	}

	if ( (chat->isVisible() == FALSE) &&
	     (Config->GetSendHidePrivateChatToPublicChat() == TRUE) )
	{
		CString nick;

		// send private chat to public chat
		if ( (msg->sFrom != msg->sFromNick) && (msg->sFromNick != "") )
			nick = msg->sFrom + "::" + msg->sFromNick;
		else
			nick = msg->sFrom;
		m_pHubChat->AddMessage( nick.Data(), msg->sString.Data(), TRUE );
	}
}

/** */
void DCClient::SetNickColor( QString nick, QColor color )
{
	DC_QCheckListItem *item;

	if ( (item = (DC_QCheckListItem *)ListView_USERLIST->findItem( nick, 0 )) != 0 )
	{
		if ( item->colorText != color )
		{
			item->colorText = color;
			item->repaint();
		}
	}
}

/** */
void DCClient::CloseChat( DCChat * chat )
{
	chat->removeEventFilter(this);

	if ( m_pChatList != 0 )
	{
		m_pChatList->Remove(chat->GetNick());
	}

	if ( m_bUseTabWidget )
	{
		TabWidget_CHAT->removePage(chat);
	}
	else
	{
		QListViewItem* item;
		if ( (item = ListView_CHATUSERLIST->findItem(chat->GetNick().Data(),0)) != 0 )
		{
			ListView_CHATUSERLIST->takeItem(item);
			delete item;
		}
	}

	chat->close();
}

/** */
void DCClient::CloseAllChats()
{
	DCChat * chat=0;

	while( m_pChatList->Next((CObject*&)chat) )
	{
		CloseChat(chat);
		chat = 0;
        }
}

/** */
void DCClient::slotDoubleClickedUserList( QListViewItem * item )
{
	if ( item == 0 )
	{
		return;
	}

	DC_PrivateChat( item->text(0), "", "", TRUE );
}

/** */
void DCClient::slotDoubleClickedChatUserList( QListViewItem * item )
{
	DCChat * chat = 0;
	QString nick;

	if ( item == 0 )
	{
		return;
	}

	nick = item->text(0);

	if ( nick == "" )
	{
		return;
	}

	if ( m_pChatList->Get( nick.ascii(), (CObject*&)chat ) == 0 )
	{
		if ( !m_bUseTabWidget )
			chat->setEnabled(TRUE);
		chat->show();

		SetNickColor(nick,Qt::black);
	}
}

/** */
int DCClient::selectedItems( QListView * list, CList<CString> * lst )
{
	QListViewItemIterator it( (QListView *)list );

	for ( ; it.current(); it++ )
	{
		if ( it.current()->isSelected() )
		{
			lst->Add(new CString( it.current()->text(0)) );
//			lst.append(it.current());
		}
	}

	return lst->Count();
}

/** */
void DCClient::slotRightButtonClickedUserList( QListViewItem *, const QPoint &, int column )
{
	int id;
	QPopupMenu *m;
	QString username;
	CList<CString> selitems;
	QListViewItem * curitem;
	CString * ps;
	int numSelected;

	numSelected = selectedItems(ListView_USERLIST,&selitems);

	m = new QPopupMenu(this);

	if ( numSelected == 1 )
	{
		curitem = ListView_USERLIST->currentItem();
		username = curitem->text(0);
		DCMenuHandler::InsertMenu( m, emiUSER_CAPTION, FALSE, tr("User: ") + username );
	}
	else
	{
		QString numUsers;
		numUsers.setNum(numSelected);
		DCMenuHandler::InsertMenu( m, emiUSER_CAPTION, FALSE, numUsers + tr(" users") );
	}

	DCMenuHandler::InsertMenu( m, emiSEPARATOR );
	DCMenuHandler::InsertMenu( m, emiPRIVATE_CHAT, (numSelected > 0) );
	DCMenuHandler::InsertMenu( m, emiBROWSE_USER_FILES, (numSelected > 0) );
	DCMenuHandler::InsertMenu( m, emiSEPARATOR );
	DCMenuHandler::InsertMenu( m, emiADD_USER_SLOT, (numSelected > 0) );
	DCMenuHandler::InsertMenu( m, emiADD_PERMANENT_USER_SLOT, (numSelected > 0) );
	DCMenuHandler::InsertMenu( m, emiREMOVE_USER_SLOT, (numSelected > 0) );
	DCMenuHandler::InsertMenu( m, emiSEPARATOR );
	DCMenuHandler::InsertMenu( m, emiUPDATE_USER, (numSelected > 0) );
	DCMenuHandler::InsertMenu( m, emiRELOAD_USERLIST, (GetConnectionState() == estCONNECTED) );
	DCMenuHandler::InsertMenu( m, emiSEPARATOR );
	DCMenuHandler::InsertMenu( m, emiCOPY_COLUMN_TO_CLIPBOARD, (numSelected == 1) );
	DCMenuHandler::InsertMenu( m, emiCOPY_ROW_TO_CLIPBOARD, (numSelected == 1) );
	DCMenuHandler::InsertMenu( m, emiSEPARATOR );
	DCMenuHandler::InsertMenu( m, emiCONNECT, (GetConnectionState() == estNONE) );
	DCMenuHandler::InsertMenu( m, emiDISCONNECT, (GetConnectionState() == estCONNECTED));
	DCMenuHandler::InsertMenu( m, emiSEPARATOR );
	DCMenuHandler::InsertMenu( m, emiKICK, IsAdmin() && (numSelected > 0) );
	DCMenuHandler::InsertMenu( m, emiFORCE_MOVE, IsAdmin() && (numSelected > 0) );

	id = m->exec(QCursor::pos());

	delete m;

	if ( id == emiPRIVATE_CHAT )
	{
		for(ps=0;(ps=selitems.Next(ps)) != 0;)
		{
			DC_PrivateChat( ps->Data(), "", "", TRUE );
		}
	}
	else if ( id == emiBROWSE_USER_FILES )
	{
		if ( GetMode() == ecmNone )
		{
			return;
		}

		for(ps=0;(ps=selitems.Next(ps)) != 0;)
		{
			/** add transfer to the waitlist */
			pTransferView->DLM_QueueAdd( ps->Data(), GetHubName(), GetIP()+":"+QString().setNum(GetPort()).ascii(),
						"MyList.DcLst", "MyList.DcLst", "", "", eltBUFFER,
						0, 0 );
		}
	}
	else if ( id == emiUPDATE_USER )
	{
		for(ps=0;(ps=selitems.Next(ps)) != 0;)
		{
			CString s = CString("$GetINFO ") + *ps + " " + GetNick() + "|";

			// send getinfo
			SendString( s );
		}
	}
	else if ( id == emiCOPY_COLUMN_TO_CLIPBOARD )
	{
		QListViewItem *item1;
		QClipboard *cb = QApplication::clipboard();
		cb->setText("");

		if ( (item1=ListView_USERLIST->currentItem()) != 0 )
		{
			cb->setText( item1->text(column) );
		}
	}
	else if ( id == emiCOPY_ROW_TO_CLIPBOARD )
	{
		int idx;
		QString s="";
		QListViewItem *item1;
		QClipboard *cb = QApplication::clipboard();
		cb->setText("");

		if ( (item1=ListView_USERLIST->currentItem()) != 0 )
		{
			for( idx = 0; idx < ListView_USERLIST->columns(); idx++ )
			{
				s += item1->text(idx) + "\n";
			}
			cb->setText(s);
		}
	}
	else if ( id == emiADD_USER_SLOT )
	{
		for(ps=0;(ps=selitems.Next(ps)) != 0;)
		{
			pTransferView->DLM_AddUserSlot(*ps, GetHubName(), 1);
		}
	}
	else if ( id == emiADD_PERMANENT_USER_SLOT )
	{
		for(ps=0;(ps=selitems.Next(ps)) != 0;)
		{
			pTransferView->DLM_AddUserSlot(*ps, GetHubName(), 0);
			pTransferView->DLM_AddUserSlot(*ps, GetHubName(), 0, TRUE);
		}
	}
	else if ( id == emiREMOVE_USER_SLOT )
	{
		for(ps=0;(ps=selitems.Next(ps)) != 0;)
		{
			pTransferView->DLM_AddUserSlot(*ps, GetHubName(), 0);
		}
	}
	else if ( id == emiKICK )
	{
		QString kickmessage;

		if ( !GetOPKickMessage(kickmessage) )
		{
			return;
		}
		
		for(ps=0;(ps=selitems.Next(ps)) != 0;)
		{
			OPKick( ps->Data(), kickmessage );
		}
	}
	else if ( id == emiFORCE_MOVE )
	{
		QString host,message;

		if ( !GetOPForceMoveMessage(message,host) )
		{
			return;
		}
				
		for(ps=0;(ps=selitems.Next(ps)) != 0;)
		{
			OPForceMove( ps->Data(), message, host );
		}
	}
	else if ( id == emiCONNECT )
	{
		Connect();
		UpdateReconnect( ersNONE, 0 );
	}
	else if ( id == emiDISCONNECT )
	{
		Disconnect();
	}
	else if ( id == emiRELOAD_USERLIST )
	{
		RequestNickList();
	}
}

/** */
bool DCClient::GetOPKickMessage( QString & message )
{
	bool ok = FALSE;

	message = QInputDialog::getText(
		tr("OP Kick"),
		tr("Please enter a reason"),
		QLineEdit::Normal, QString::null, &ok, 0 );

	return ok;
}

/** */
bool DCClient::GetOPForceMoveMessage( QString & message, QString & host )
{
	bool ok = FALSE;

	host = QInputDialog::getText(
		tr("OP Force Move"),
		tr("Please enter host"),
		QLineEdit::Normal, QString::null, &ok, 0 );

	if ( !ok )
	{
		return ok;
	}

	ok = FALSE;

	message = QInputDialog::getText(
		tr("OP Force Move"),
		tr("Please enter a message"),
		QLineEdit::Normal, QString::null, &ok, 0 );

	return ok;
}

/** */
bool DCClient::OPKick( QString nick, QString message )
{
	bool res = FALSE;

	SendPrivateMessage( GetNick(), nick.ascii(), ("You Are Being kicked Because: " + message).ascii());
	SendChat( GetNick(), GetNick() + " is kicking " + nick.ascii() + " Because: " + message.ascii() );
	SendKick(nick.ascii());

	return res;
}

/** */
bool DCClient::OPForceMove( QString nick, QString message, QString host )
{
	bool res = FALSE;

	SendOpForceMove( nick.ascii(), host.ascii(), message.ascii() );

	return res;
}

/** */
void DCClient::slotDoubleClickedUserChat( QListBoxItem * item )
{
	DCChat * chat;
	QString nick;

	if ( item == 0 )
	{
		return;
	}

	nick = item->text();

	if ( nick == "" )
	{
		return;
	}

	if ( m_pChatList->Get( nick.ascii(), (CObject*&)chat ) == 0 )
	{
		if ( !m_bUseTabWidget )
			chat->setEnabled(TRUE);
		chat->show();
		SetNickColor(nick,Qt::black);
        }
}
