/***************************************************************************
                          dctranslator.cpp  -  description
                             -------------------
    begin                : Die Nov 19 2002
    copyright            : (C) 2002 by Mathias Kster
    email                : mathen@ketelhot.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <qapplication.h>
#include <qhttp.h>
#include <qnetwork.h>
#include <qurloperator.h>
#include <qregexp.h>
#include <qurl.h>
#include <qinputdialog.h>

#include <dcgui.h>
#include <dcevent.h>

#include "dctranslator.h"

DCTranslator * gDCTranslator = 0;

DCTranslator::DCTranslator(QWidget * /*parent*/, const char * /*name*/ ) // : QWidget(parent,name) {
{
	qInitNetworkProtocols();

	pOp = new QUrlOperator();

	InitDocument();
}

DCTranslator::~DCTranslator()
{
	if ( pOp )
	{
		delete pOp;
	}
}

/** */
void DCTranslator::InitDocument()
{
	m_sLanguage = "en_de";

	connect( pOp, SIGNAL(data(const QByteArray &, QNetworkOperation *)), this, SLOT(slotData(const QByteArray &, QNetworkOperation *)) );
	connect( pOp, SIGNAL(dataTransferProgress(int, int, QNetworkOperation*)), this, SLOT(slotDataTransferProgress( int, int, QNetworkOperation *)) );
	connect( pOp, SIGNAL(finished( QNetworkOperation *)), this, SLOT(slotFinished( QNetworkOperation *)) );
}

/** */
QString DCTranslator::SelectLanguage()
{
	QStringList lst;

	lst << "en_de" << "en_fr" << "en_zh" << "en_it" << "en_ja" << "en_ko"
		<< "en_pt" << "en_es" << "zh_en" << "fr_en" << "fr_de" << "de_en"
		<< "de_fr" << "it_en" << "ja_en" << "ko_en" << "pt_en" << "ru_en"
		<< "es_en";

	bool ok = FALSE;
	QString res = QInputDialog::getItem( tr("Translator"), tr("Choose a language"), lst, 0, TRUE, &ok, 0 );

	if ( ok )
		return res;
	else
		return "";
}

/** */
bool DCTranslator::Translate( QObject * sender, QString lang, QString text )
{
	DC_TranslationEvent * TranslationEvent=0;
	QString url,tr_text;

	// convert text to correct html ... 
	tr_text = text;

	QUrl::encode(tr_text);

	url  = "http://babelfish.altavista.com/babelfish/tr?doit=done&tt=urltext&intl=1";
	url += "&urltext=" + tr_text;
	url += "&lp=";

	if ( lang == "" )
		url += m_sLanguage;
	else
		url += lang;

	if ( m_UrlList.Get( url.ascii(), (CObject *&)TranslationEvent ) == 0 )
	{
		return FALSE;
	}

	TranslationEvent = new 	DC_TranslationEvent(EVENT_TRANSLATION);

	TranslationEvent->m_bTranslate   = FALSE;
	TranslationEvent->m_pReceiver    = sender;
	TranslationEvent->m_sOriginal    = text;
	TranslationEvent->m_sTranslation = "";

	m_UrlList.Add( url.ascii(), (CObject*)TranslationEvent );

	pOp->get(url);

	return TRUE;
}

/** */
void DCTranslator::slotData( const QByteArray & data, QNetworkOperation * no )
{
	DC_TranslationEvent * TranslationEvent=0;

	if ( m_UrlList.Get( no->arg(0).ascii(), (CObject *&)TranslationEvent ) != 0 )
	{
		printf("URL NOT FOUND: %s\n",no->arg(0).ascii());
		return;
	}

	TranslationEvent->m_sTranslation += data;
}

/** */
void DCTranslator::slotDataTransferProgress( int bytesDone, int bytesTotal, QNetworkOperation * )
{
	printf("%d/%d\n",bytesDone,bytesTotal);
//	QString s;
//	s.sprintf("%d/%d",bytesDone,bytesTotal);
//	TextLabel_Status->setText(s);
}

/** */
void DCTranslator::slotFinished( QNetworkOperation * op )
{
	DC_TranslationEvent * TranslationEvent=0;

	if ( m_UrlList.Get( op->arg(0).ascii(), (CObject *&)TranslationEvent ) != 0 )
	{
		printf("URL NOT FOUND: %s\n",op->arg(0).ascii());
		return;
	}

	printf("STATE: %d\n",op->state());

	switch( op->state() )
	{
		case QNetworkProtocol::StWaiting:
			return;
			break;
		case QNetworkProtocol::StInProgress:
			return;
			break;
		case QNetworkProtocol::StDone:
		{
			QString s,s1;
			int i1,i2;
			s1 = "<Div style=padding:10px;>";
			s = TranslationEvent->m_sTranslation;
			if ( (i1 = s.find(s1)) == -1 )
				TranslationEvent->m_sTranslation = "";
			else if ( (i2 = s.find("</div>",i1)) == -1 )
				TranslationEvent->m_sTranslation = "";
			else
			{
				TranslationEvent->m_sTranslation = s.mid(i1+s1.length(),i2-i1-s1.length());
				// remove 3 0x0a at end
				TranslationEvent->m_sTranslation = TranslationEvent->m_sTranslation.left(TranslationEvent->m_sTranslation.length()-3);
				TranslationEvent->m_bTranslate = TRUE;
			}
			printf("translate: '%s'\n",TranslationEvent->m_sTranslation.ascii());

			break;
		}

//		case QNetworkProtocol::StFailed:
//		case QNetworkProtocol::StStopped:
		default:
			TranslationEvent->m_sTranslation = "";
			TranslationEvent->m_bTranslate = FALSE;
			break;
	}

	m_UrlList.Remove(op->arg(0).ascii());

	QApplication::postEvent( TranslationEvent->m_pReceiver, TranslationEvent );
}
