/***************************************************************************
                          main.cpp  -  description
                             -------------------
    begin                : Mon Okt  1 19:59:45 CEST 2001
    copyright            : (C) 2001 by Mathias Kster
    email                : mathen@ketelhot.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <dclib/dcos.h>

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#undef NDEBUG
#include <assert.h>

#ifndef WIN32
#include <stdlib.h>
#endif

#include <qapplication.h>
#include <qfont.h>
#include <qstring.h>
#include <qtextcodec.h>
#include <qtranslator.h>
#include <qlabel.h>
#include <qstylefactory.h>
#include <private/qinternal_p.h>

#include "dcconfig.h"
#include "dcsplash.h"
#include "dcgui.h"
#include <dcpluginmanager.h>
#include <dcserverlist.h>
#include <dctransferview.h>

#include <dclib/casyncdns.h>
#include <dclib/cmanager.h>
#include <dclib/cquerymanager.h>
#include <dclib/cfilemanager.h>

#ifdef HAVE_SOCKS
extern "C" {
#include <socks.h>
}
#endif

#ifdef HAVE_SIGCONTEXT
#include <signal.h>

CString arg_0 = "";

inline void addr2line( unsigned long ebp )
{
	char b[200];
	CString s;

	if ( ebp == 0 )
	{
		return;
	}

	sprintf(b," 0x%lx",ebp);
	s  = "addr2line -Cfe " + arg_0 + b;

	if ( system(s.Data()) != 0 )
	{
		printf("addr2line failed !\n");
	}
}

/** */
inline void z_dump_backtrace(unsigned long eip, unsigned long first_ebp)
{
	/* NOTE: this is i386 specific */
	unsigned long *ebp;

	fprintf(stderr, "--- retaddr=0x%lx, ebp=0x%lx\n", eip, first_ebp);
	addr2line(eip);

	ebp = (unsigned long *) first_ebp;
	while (ebp > (unsigned long *) &ebp && *ebp)
	{
		fprintf(stderr, "--- retaddr=0x%lx, ebp=0x%lx\n", *(ebp+1), *ebp);
		addr2line(*(ebp+1));
		ebp = (unsigned long *) *ebp;
	}
}

#ifdef HAVE_SIGCONTEXT_LINUX
/** */
void sigsegv_handler(int signo)
{
	struct sigcontext *p = (struct sigcontext *) (((char *) &p) + 16);

	fprintf(stderr, "Signal (%d) received, stackdump follows; eax='%08lx', ebx='%08lx', ecx='%08lx', edx='%08lx', eip='%08lx'\n",
        	signo,p->eax,p->ebx,p->ecx,p->edx,p->eip);
	z_dump_backtrace(p->eip, p->ebp);
	exit(1);
}
#elif HAVE_SIGCONTEXT_FREEBSD
/** */
void sigsegv_handler(int signo)
{
	struct sigcontext *p = (struct sigcontext *) (((char *) &p) + 16);

	fprintf(stderr, "Signal (%d) received, stackdump follows; eax='%08lx', ebx='%08lx', ecx='%08lx', edx='%08lx', eip='%08lx'\n",
        	signo,p->sc_eax,p->sc_ebx,p->sc_ecx,p->sc_edx,p->sc_eip);
	z_dump_backtrace(p->sc_eip, p->sc_ebp);
	exit(1);
}
#else
/** */
void sigsegv_handler(int signo)
{
	fprintf(stderr, "Signal (%d) received, sorry no backtrace\n", \
        	signo);
	exit(1);
}
#endif
#endif

/** */
int main(int argc, char *argv[])
{
	int i;
	int ret;
	CString configpath = "";
	QApplication * pApplication;

	// startup parameter
	bool bThemeSupport  = TRUE;
	bool bSplash = TRUE;
#ifdef HAVE_SIGCONTEXT
	bool bBacktrace = TRUE;

	arg_0 = argv[0];
#endif

#ifdef HAVE_SOCKS
	SOCKSinit(argv[0]);
#endif

	// check xml version
	LIBXML_TEST_VERSION;

	// check for QSharedDoubleBuffer
	printf("Checking qt version ...\n");
	printf("Compiled for '%s' using '%s'\n",QT_VERSION_STR,qVersion());
	fflush(stdout);
	printf("Checking qt privates ...\n");fflush(stdout);
	QSharedDoubleBuffer * buffer = new QSharedDoubleBuffer();
	delete buffer;
	printf("private test ok !\n");fflush(stdout);

	// ignore pipe signal
#ifndef WIN32
	signal(SIGPIPE, SIG_IGN);
#endif

	// parameter stuff ...
	for(i=1;i<argc;i++)
	{
		if ( QString(argv[i]) == "--disable-splash" )
			bSplash = FALSE;
		else if ( QString(argv[i]) == "--disable-theme" )
			bThemeSupport = FALSE;
		else if ( QString(argv[i]) == "--check-theme" )
		{
			// check buggy style/theme qt support
			printf("Check theme support ... ");
			QStringList list = QStyleFactory::keys();
			printf("ok %d styles found\n",list.count());
			return 0;
		}
#ifdef HAVE_SIGCONTEXT
		else if ( QString(argv[i]) == "--disable-backtrace" )
			bBacktrace = FALSE;
#endif
		else if ( QString(argv[i]) == "-c" )
		{
			i++;
			if ( i<argc )
			{
				configpath = argv[i];
			}
			else
			{
				printf("Wrong parameter !\n");
				return -1;
			}
		}
		else
		{
			printf("Wrong parameter !\n");
			return -1;
		}
	}

#ifdef HAVE_SIGCONTEXT
	if(bBacktrace)
		signal(SIGSEGV, sigsegv_handler);
#endif
	pApplication = new QApplication(argc, argv);

	DCSplash * s;
	s = new DCSplash(0);
	assert(s);
	if (bSplash)
		s->show();
	else
		s->hide();

	ret = 0;

	CManager::SetInstance(new CManager);
	CAsyncDns::SetInstance(new CAsyncDns);

	assert(new DCConfig(configpath));

	Config->SetThemeSupport(bThemeSupport);

	s->TextLabel_STATUS->setText("Load LIB-Config ...");
	pApplication->processEvents();

	if ( Config->LoadDCLib() != 0 )
		ret = -1;

	// create the filemanager
	s->TextLabel_STATUS->setText("Init Filemanager ...");
	pApplication->processEvents();

	CFileManager::SetInstance(new CFileManager());

	s->TextLabel_STATUS->setText("Load Hub-Profiles ...");
	pApplication->processEvents();

	Config->LoadHubProfile();

	s->TextLabel_STATUS->setText("Load Hub-List ...");
	pApplication->processEvents();

	Config->LoadDCHub();

	s->TextLabel_STATUS->setText("Load GUI-Config ...");
	pApplication->processEvents();

	if ( Config->LoadDCGui() != 0 )
		ret = -1;

	// create the querymanager
	CQueryManager::SetInstance(new CQueryManager());

	QFont appFont;
	if (appFont.fromString( QString(Config->GetAppFont().Data()) ) == FALSE)
	{
		// In case the xml file is messed up
		appFont.fromString(DEFAULT_APP_FONT);
	}
	pApplication->setFont(appFont);

	s->TextLabel_STATUS->setText("Install translator ...");
	pApplication->processEvents();

	QTranslator tor(0);

	if ( Config->GetLanguageFile() != "" )
	{
		tor.load( Config->GetLanguageFile().Data(), "." );
	}
	else
	{
		tor.load( QString("dcgui.") + QTextCodec::locale(), "." );
	}

	pApplication->installTranslator( &tor );

	s->TextLabel_STATUS->setText("Start ...");
	pApplication->processEvents();

	DCGuiApp *dcgui = new DCGuiApp();

	assert(dcgui);

	s->TextLabel_STATUS->setText("Load plugins ...");
	pApplication->processEvents();

	assert(new DCPluginManager());
	pPluginManager->Load();

	s->TextLabel_STATUS->setText("Init plugins ...");
	pApplication->processEvents();

	pPluginManager->InitPlugins();

	s->TextLabel_STATUS->setText("Start ...");
	pApplication->processEvents();

	if ( ret == 0 )
	{
		s->TextLabel_STATUS->setText("Load sharelist ...");
		pApplication->processEvents();

		Config->LoadDCMyList();
	}

	// hide splash
	s->hide();
	pApplication->processEvents(10);
	delete s;

	if ( ret != 0 )
	{
		// open options dialog ...
		dcgui->ShowOptionsDialog();
	}

	pApplication->setMainWidget(dcgui);
	pApplication->processEvents(10);

	// load download manager queue
	pTransferView->DLM_LoadQueue();
	pApplication->processEvents(10);

	dcgui->show();
	pApplication->processEvents(10);

	dcgui->initWindows();
	pApplication->processEvents(10);

	// connect to hubs with the auto connect flag
	pServerList->AutoConnect();
	pApplication->processEvents();

	ret = pApplication->exec();

	pPluginManager->DeInitPlugins();

	delete pPluginManager;

	if ( CFileManager::Instance() )
	{
		CFileManager::Instance()->Stop();
		CFileManager::Instance()->SaveIndex();
		delete CFileManager::Instance();
	}

	if ( CQueryManager::Instance() )
	{
		delete CQueryManager::Instance();
	}

	if ( CAsyncDns::Instance() )
	{
		delete CAsyncDns::Instance();
	}

	pApplication->setMainWidget(0);
	delete dcgui;
	delete pApplication;

	if( Config )
	{
		delete Config;
		Config = 0;
	}

	if ( CManager::Instance() )
	{
		delete CManager::Instance();
	}

	printf("application exit ok %d\n",ret);

#ifdef HAVE_SIGCONTEXT
	if(bBacktrace)
		signal(SIGSEGV, 0);
#endif

#ifdef WIN32
#ifdef _DEBUG
	_CrtDumpMemoryLeaks();
#endif
#endif

	return ret;
}
